'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.swaggerError = exports.getSwaggerClient = exports.rejectUnexistingVersion = exports.unfoldMaybeLibQuery = exports.stringifyLibQuery = exports.isLibQueryValid = exports.getLibName = exports.parseLibQuery = exports.defaultToLatest = exports.prependVIfNeeded = undefined;

var _ramda = require('ramda');

var R = _interopRequireWildcard(_ramda);

var _swaggerClient = require('swagger-client');

var _swaggerClient2 = _interopRequireDefault(_swaggerClient);

var _ramdaFantasy = require('ramda-fantasy');

var _xodFuncTools = require('xod-func-tools');

var _errorCodes = require('./errorCodes');

var ERR_CODES = _interopRequireWildcard(_errorCodes);

var _messages = require('./messages');

var MSG = _interopRequireWildcard(_messages);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

const LATEST = 'latest';

// :: Nullable String -> String
const prependVIfNeeded = exports.prependVIfNeeded = R.when(R.allPass([_xodFuncTools.notNil, _xodFuncTools.notEmpty, R.compose(R.complement(R.equals('v')), R.nth(0)), R.complement(R.equals(LATEST))]), R.concat('v'));

// :: Nullable String -> String
const defaultToLatest = exports.defaultToLatest = R.when(R.anyPass([R.isEmpty, R.isNil]), R.always(LATEST));

// :: String -> Maybe LibQueryParams
const parseLibQuery = exports.parseLibQuery = R.compose(R.map(R.applySpec({
  owner: R.nth(0),
  libname: R.nth(1),
  version: R.compose(defaultToLatest, prependVIfNeeded, R.nth(2))
})), R.ifElse(R.compose(R.gt(R.__, 1), R.length), _ramdaFantasy.Maybe.of, _ramdaFantasy.Maybe.Nothing), R.reject(R.isEmpty), R.tail, R.match(/^([^@/]+?)\/([^@/]+?)(?:@([^@/]+?))?$/));

// :: String -> LibName
// It parses libQuery ("xod/core" or "xod/core@0.11.0")
// and returns lib name without version ("xod/core").
const getLibName = exports.getLibName = libQuery => R.compose(qp => `${qp.owner}/${qp.libname}`, (0, _xodFuncTools.explodeMaybe)(`Expected correct library name format, like "xod/core@0.11.0", but got "${libQuery}"`), parseLibQuery)(libQuery);

// :: String -> Boolean
const isLibQueryValid = exports.isLibQueryValid = R.compose(_ramdaFantasy.Maybe.isJust, parseLibQuery);

// :: LibQueryParams -> LibName
const stringifyLibQuery = exports.stringifyLibQuery = ({ owner, libname, version }) => `${owner}/${libname}@${version}`;

// :: (LibQueryParams -> a) -> Maybe LibQueryParams -> Promise a Error
const unfoldMaybeLibQuery = exports.unfoldMaybeLibQuery = R.curry((justFn, maybe) => (0, _xodFuncTools.maybeToPromise)(() => (0, _xodFuncTools.rejectWithCode)(ERR_CODES.CANT_PARSE_LIBRARY_REQUEST, new Error(MSG.CANT_PARSE_LIBRARY_REQUEST)), justFn, maybe));

// :: LibParams -> LibData -> Promise LibData Error
const rejectUnexistingVersion = exports.rejectUnexistingVersion = R.curry((params, libdata) => R.ifElse(R.propEq('version', LATEST), () => Promise.resolve(libdata), () => R.ifElse(R.compose(R.contains(params.version), R.prop('versions')), Promise.resolve.bind(Promise), () => (0, _xodFuncTools.rejectWithCode)(ERR_CODES.CANT_FIND_LIB_VERSION, new Error(MSG.cantFindLibVersion(params))))(libdata))(params));

// Create memoized function to prevent fetching swagger URL
// on each fetch request
const getSwaggerClient = exports.getSwaggerClient = (() => {
  const memo = {};

  return url => {
    const index = JSON.stringify(url);

    if (index in memo) {
      return memo[index];
    }

    const client = (0, _swaggerClient2.default)(url).catch(() => {
      delete memo[index];
      return (0, _xodFuncTools.rejectWithCode)(ERR_CODES.CANT_GET_SWAGGER, new Error(MSG.SERVICE_UNAVAILABLE));
    });

    return memo[index] = client;
  };
})();

// TODO: duplicate in xod-cli
const swaggerError = exports.swaggerError = err => {
  if (err instanceof Error) {
    return Object.assign(new Error(MSG.SERVICE_UNAVAILABLE), {
      errorCode: ERR_CODES.SERVICE_UNAVAILABLE
    });
  }

  const { response, status } = err;
  let res;
  if (response.body && response.body.originalResponse) {
    res = JSON.parse(response.body.originalResponse);
  } else if (response.body) {
    res = response.body;
  } else {
    res = response;
  }

  if (status === 400) {
    return new Error(R.pathOr('Bad request', ['errors', 0, 'errors', 0, 'message'], res));
  }

  return new Error(`${status} ${JSON.stringify(res, null, 2)}`);
};